package edu.umd.dawn.common.annotations;

import edu.umd.dawn.common.enums.Role;
import edu.umd.dawn.common.exceptions.BaseExceptions;
import edu.umd.dawn.common.exceptions.DawnException;
import edu.umd.dawn.common.jwt.Claims;
import edu.umd.dawn.common.services.UserAuthService;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.log4j.Log4j2;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Aspect
@Component
@Log4j2
public class RoleRestrictionAspect extends AspectBase<RoleRestriction> {

    @Autowired
    private UserAuthService service;

    @Value("${config.local}")
    private boolean local = false;

    @Around("@annotation(RoleRestriction) && within(edu.umd.dawn..**)")
    public Object run(ProceedingJoinPoint jointPoint) throws Exception, Throwable {
        RoleRestriction annotation = grabAnnotation(jointPoint, RoleRestriction.class);
        HttpServletRequest request = grabRequest();
        if (!local) {
            Claims claims = (Claims) request.getAttribute("claims");
            if (claims == null) {
                throw new DawnException(BaseExceptions.FORBIDDEN);
            }
            Role role = service.getUserRole(claims.getUserId());
            if (!role.isAccessAllowed(annotation.value())) {
                throw new DawnException(BaseExceptions.FORBIDDEN);
            }
        } else if (!annotation.disableWarning()) {
            log.warn("RoleRestriction annotation has been disabled - if this is a production environment, consider this"
                    + " a critical security error. Request with unknown role accessed endpoint with restriction "
                    + annotation.value().toString() + ".");
        }

        Object proceed = jointPoint.proceed();
        return proceed;
    }
}
