package edu.umd.dawn.common.filters;

import com.google.common.base.CaseFormat;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

/**
 * This filter will convert any underscore case variable to camelcase.
 * It is intended to support backwards compatibility with golang requests which had variables written
 * like user_id instead of userId.
 */
@Log4j2
@Component
public class CaseInsensitiveFilter extends OncePerRequestFilter {

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {
        Map<String, String[]> formattedParams = new ConcurrentHashMap<>();

        List<String> invalidParameterCase = new ArrayList<>();

        for (String param : request.getParameterMap().keySet()) {
            String formattedParam = param;
            if (param.contains("_")) {
                invalidParameterCase.add(param);
                formattedParam = CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, param);
            }
            formattedParams.put(formattedParam, request.getParameterValues(param));
        }

        filterChain.doFilter(new CaseInsensitiveFilterWrapper(request, formattedParams), response);

        if (invalidParameterCase.size() > 0) {
            log.warn(String.format(
                    "client is still using deprecated naming scheme for parameter(s) %s",
                    invalidParameterCase.toString()));
        }
    }
}
