package edu.umd.dawn.common.interceptor;

import edu.umd.dawn.common.jwt.Claims;
import edu.umd.dawn.common.jwt.JWTUtil;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

/**
 * interceptor to pull the claims and token for a jwt
 * @implNote This does NOT throw any error if a jwt is not provided
 */
@Log4j2
@Component
public class JWTInterceptor implements HandlerInterceptor {

    @Value("${config.serviceName:dawn-service}")
    private String serviceName;

    @Value("${config.local:false}")
    private boolean local;

    @Value("${config.jwt.warn:true}")
    private boolean warn;

    @Value("${config.accessSecret:empty}")
    private String accessSecret;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {

        if (accessSecret.equals("empty")) {
            log.warn("This service has not been provided the configuration value for `config.accessSecret` in the"
                    + " application.properties configuration. Since it uses the JWTInterceptor, this is a required"
                    + " value for proper functionality.");
        }

        if (!local) {
            String token = request.getHeader("Authorization");
            if (token != null && !token.equals("")) {
                if (token.startsWith("Bearer ")) {
                    token = token.replace("Bearer ", "");
                }

                Claims claims = JWTUtil.parse(accessSecret, token).getClaims();
                request.setAttribute("claims", claims);
                request.setAttribute("token", token);
            } else if (warn) {
                log.warn("No jwt provided");
            }
        } else {
            log.warn("JWT interceptor has been disabled - if this is a production environment, consider this a"
                    + " critical security error");
        }
        return true;
    }
}
